#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <set>

typedef long long llong;
const int MAXN = 400000 + 10;
const int INF  = 1e9;

int n;
int a[MAXN];
int ld[MAXN];
int dep[MAXN];
int sz[MAXN];
int dp[MAXN];
int dp2[MAXN];
bool in[MAXN];
std::vector <int> g[MAXN];
std::vector <int> divs[MAXN];
std::multiset <int> maxDepth[MAXN];
int answer;

void buildDFS(int node, int par)
{
    for (int &u : g[node])
    {
        if (u == par)
        {
            std::swap(u, g[node].back());
            g[node].pop_back();
            break;
        }
    }

    sz[node] = 1;
    dep[node] = dep[par] + 1;
    for (const int &u : g[node])
    {
        buildDFS(u, node);
        sz[node] += sz[u];
    }

    for (int i = 1 ; i < g[node].size() ; ++i)
    {
        if (sz[g[node][i]] > sz[g[node][0]])
        {
            std::swap(g[node][i], g[node][0]);
        }
    }
}

void add(int node, int lcaDepth)
{
    for (const int &d : divs[node])
    {
        if (maxDepth[d].size())
        {
            answer = std::max(answer, (*maxDepth[d].rbegin()) + dep[node] - 2 * lcaDepth);
        }

        maxDepth[d].insert(dep[node]);
    }

    for (const int &u : g[node])
    {
        add(u, lcaDepth);
    }
}

void remove(int node)
{
    for (const int &d : divs[node])
    {
        assert(maxDepth[d].count(dep[node]));
        maxDepth[d].erase(maxDepth[d].find(dep[node]));
    }

    for (const int &u : g[node])
    {
        remove(u);
    }
}

void dfs(int node)
{
    for (int i = 1 ; i < g[node].size() ; ++i)
    {
        dfs(g[node][i]);
        remove(g[node][i]);
    }

    if (g[node].size())
    {
        dfs(g[node][0]);
    }

    for (int i = 1 ; i < g[node].size() ; ++i)
    {
        add(g[node][i], dep[node]);
    }

    for (const int &d : divs[node])
    {
        if (maxDepth[d].size())
        {
            answer = std::max(answer, (*maxDepth[d].rbegin()) - dep[node]);
        }

        maxDepth[d].insert(dep[node]);
    }
}

void solve()
{
    std::fill(ld + 1, ld + MAXN, MAXN);
    for (int i = 2 ; i < MAXN ; ++i)
    {
        if (ld[i] == MAXN)
        {
            for (llong j = i ; j < MAXN ; j += i)
            {
                ld[j] = std::min(ld[j], i);
            }
        }
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        int lastD = -1;
        while (a[i] > 1)
        {
            if (lastD != ld[a[i]])
            {
                divs[i].push_back(ld[a[i]]);
                lastD = ld[a[i]];
            }

            a[i] /= ld[a[i]];
        }
    }

    buildDFS(1, 0);
    dfs(1);

    std::cout << answer << '\n';
}

void input()
{
    std::cin >> n;
    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> a[i];
    }

    for (int i = 1 ; i < n ; ++i)
    {
        int u, v;
        std::cin >> u >> v;
        assert(u != v);
        g[u].push_back(v);
        g[v].push_back(u);
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();

    return 0;
}